# -*- coding: utf-8 -*-

import logging
import os
import unittest

import six
from requests import HTTPError
from six.moves import configparser

from shipane_sdk import Client

if six.PY2:
    ConfigParser = configparser.RawConfigParser
else:
    ConfigParser = configparser.ConfigParser


class ClientManagementTest(unittest.TestCase):
    def setUp(self):
        logging.basicConfig(level=logging.DEBUG)
        config = ConfigParser()
        dir_path = os.path.dirname(os.path.realpath(__file__))
        config.read('{}/../config/config.ini'.format(dir_path))
        self.client = Client(logging.getLogger(), host=config.get('ShiPanE', 'host'), key=config.get('ShiPanE', 'key'))

    def test_start_clients(self):
        try:
            self.client.start_clients()
        except HTTPError as e:
            self.fail()

    def test_shutdown_clients(self):
        try:
            self.client.shutdown_clients()
        except HTTPError as e:
            self.fail()



# -*- coding: utf-8 -*-

from datetime import datetime

import requests

from shipane_sdk.base_quant_client import BaseQuantClient
from shipane_sdk.joinquant.transaction import JoinQuantTransaction


class JoinQuantClient(BaseQuantClient):
    BASE_URL = 'https://www.joinquant.com'

    def __init__(self, **kwargs):
        super(JoinQuantClient, self).__init__('JoinQuant')

        self._session = requests.Session()
        self._username = kwargs.get('username', None)
        self._password = kwargs.get('password', None)
        self._backtest_id = kwargs.get('backtest_id', None)
        self._timeout = kwargs.pop('timeout', (5.0, 10.0))

    def login(self):
        self._session.headers = {
            'Accept': 'application/json, text/javascript, */*; q=0.01',
            'Accept-Encoding': 'gzip, deflate, br',
            'Accept-Language': 'en-US,en;q=0.8',
            'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/54.0.2840.100 Safari/537.36',
            'Referer': '{}/user/login/index'.format(self.BASE_URL),
            'X-Requested-With': 'XMLHttpRequest',
            'Origin': self.BASE_URL,
            'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
        }
        self._session.get(self.BASE_URL, timeout=self._timeout)
        response = self._session.post('{}/user/login/doLogin?ajax=1'.format(self.BASE_URL), data={
            'CyLoginForm[username]': self._username,
            'CyLoginForm[pwd]': self._password,
            'ajax': 1
        }, timeout=self._timeout)
        self._session.headers.update({
            'cookie': response.headers['Set-Cookie']
        })

        super(JoinQuantClient, self).login()

    def query(self):
        today_str = datetime.today().strftime('%Y-%m-%d')
        response = self._session.get('{}/algorithm/live/transactionDetail'.format(self.BASE_URL), params={
            'backtestId': self._backtest_id,
            'date': today_str,
            'ajax': 1
        }, timeout=self._timeout)
        transaction_detail = response.json()
        raw_transactions = transaction_detail['data']['transaction']
        transactions = []
        for raw_transaction in raw_transactions:
            transaction = JoinQuantTransaction(raw_transaction).normalize()
            transactions.append(transaction)

        return transactions



# -*- coding: utf-8 -*-

from datetime import datetime

from shipane_sdk.transaction import Transaction


class RiceQuantTransaction(object):
    def __init__(self, json):
        self.__dict__.update(json)

    def normalize(self):
        transaction = Transaction()
        transaction.completed_at = datetime.strptime(self.time, '%Y-%m-%d %H:%M:%S')
        transaction.action = 'BUY' if self.quantity > 0 else 'SELL'
        transaction.symbol = self.order_book_id
        transaction.price = self.price
        transaction.amount = abs(self.quantity)
        return transaction



# -*- coding: utf-8 -*-

from datetime import datetime

from requests import HTTPError

from shipane_sdk.jobs.basic_job import BasicJob
from shipane_sdk.market_utils import MarketUtils


class OnlineQuantFollowingJob(BasicJob):
    def __init__(self, shipane_client, quant_client, client_aliases=None, name=None, **kwargs):
        super(OnlineQuantFollowingJob, self).__init__(name, kwargs.get('schedule', None), kwargs.get('enabled', False))

        self._shipane_client = shipane_client
        self._quant_client = quant_client
        self._client_aliases = client_aliases
        self._name = name
        self._start_datatime = datetime.now()
        self._processed_transactions = []

    def __call__(self):
        if MarketUtils.is_closed():
            self._logger.warning("********** 休市期间不跟单 **********")
            if self._processed_transactions:
                del self._processed_transactions[:]
            return

        if not self._quant_client.is_login():
            self._logger.info("登录 %s", self._quant_client.name)
            self._quant_client.login()

        self._logger.info("********** 开始跟单 **********")
        try:
            all_transactions = self._quant_client.query()
            self._logger.info("获取到 %d 条委托", len(all_transactions))

            transactions = []
            for transaction in all_transactions:
                if self._is_expired(transaction):
                    continue
                transactions.append(transaction)
            self._logger.info("获取到 %d 条有效委托", len(transactions))

            for client_alias in self._client_aliases:
                client = self._client_aliases[client_alias]
                for tx in transactions:
                    try:
                        self._processed_transactions.append(tx)
                        self._logger.info("开始在[%s(%s)]以 %f元 %s %d股 %s",
                                          client_alias, client, tx.price, tx.get_cn_action(), tx.amount, tx.symbol)
                        self._shipane_client.execute(client,
                                                     action=tx.action,
                                                     symbol=tx.symbol,
                                                     type='LIMIT',
                                                     price=tx.price,
                                                     amount=tx.amount)
                    except HTTPError as e:
                        self._logger.exception("下单异常")
        except Exception as e:
            self._logger.exception("跟单异常")
        self._logger.info("********** 结束跟单 **********\n")

    @property
    def name(self):
        return self._name

    def _is_expired(self, transaction):
        if transaction.completed_at < self._start_datatime:
            return True
        if transaction in self._processed_transactions:
            return True
        return False



# -*- coding: utf-8 -*-


class Transaction(object):
    def __init__(self, **kwargs):
        self._completed_at = kwargs.get('completed_at')
        self._action = kwargs.get('action')
        self._symbol = kwargs.get('symbol')
        self._price = kwargs.get('price')
        self._amount = kwargs.get('amount')

    def __eq__(self, other):
        if self.completed_at != other.completed_at:
            return False
        if self.action != other.action:
            return False
        if self.symbol != other.symbol:
            return False
        if self.price != other.price:
            return False
        if self.amount != other.amount:
            return False
        return True

    def get_cn_action(self):
        return u'买入' if self.action == 'BUY' else u'卖出'

    @property
    def completed_at(self):
        return self._completed_at

    @completed_at.setter
    def completed_at(self, value):
        self._completed_at = value

    @property
    def action(self):
        return self._action

    @action.setter
    def action(self, value):
        self._action = value

    @property
    def symbol(self):
        return self._symbol

    @symbol.setter
    def symbol(self, value):
        self._symbol = value

    @property
    def price(self):
        return self._price

    @price.setter
    def price(self, value):
        self._price = value

    @property
    def amount(self):
        return self._amount

    @amount.setter
    def amount(self, value):
        self._amount = value



import shipane_sdk


# 初始化函数，设定要操作的股票、基准等等
def initialize(context):
    # 定义一个全局变量, 保存要操作的股票
    # 000001(股票:平安银行)
    g.security = '000001.XSHE'
    # 设定沪深300作为基准
    set_benchmark('000300.XSHG')


def process_initialize(context):
    # 创建 JoinQuantExecutor 对象
    # 可选参数包括：host, port, key, client, timeout 等
    # 请将下面的 IP 替换为实际 IP
    g.__executor = shipane_sdk.JoinQuantExecutor(
        host='xxx.xxx.xxx.xxx',
        port=8888,
        key='',
        client=''
    )


# 每个单位时间(如果按天回测,则每天调用一次,如果按分钟,则每分钟调用一次)调用一次
def handle_data(context, data):
    # 保存 order 对象
    order_ = order(g.security, 100)
    # 实盘易依据聚宽的 order 对象下单
    g.__executor.execute(order_)

    order_ = order(g.security, -100)
    g.__executor.execute(order_)

    # 撤单
    g.__executor.cancel(order_)



import shipane_sdk


# 注意：需将回测调成分钟级别
# 注意：SDK内部取今日时间来获取新股数据
# 注意：用于回测没有意义，需挂到“模拟交易”

# 在这个方法中编写任何的初始化逻辑。context对象将会在你的算法策略的任何方法之间做传递。
def init(context):
    # 每天的开市后10分钟进行新股申购。参数设置见：https://www.ricequant.com/api/python/chn#scheduler
    scheduler.run_daily(purchase_new_stocks, time_rule=market_open(minute=10))


def before_trading(context):
    # 创建 RiceQuantExecutor 对象
    # 可选参数包括：host, port, key, client, timeout 等
    # 请将下面的 IP 替换为实际 IP
    context.__executor = shipane_sdk.RiceQuantExecutor(
        host='xxx.xxx.xxx.xxx',
        port=8888,
        key='',
        client=''
    )


def purchase_new_stocks(context, bar_dict):
    if context.run_info.run_type == RUN_TYPE.PAPER_TRADING:
        context.__executor.purchase_new_stocks()
    else:
        logger.info('回测中不进行新股申购')



# -*- coding: utf-8 -*-
import json
import os
import unittest
from datetime import datetime

from shipane_sdk.joinquant.transaction import JoinQuantTransaction


class TransactionTest(unittest.TestCase):
    def setUp(self):
        dir_path = os.path.dirname(os.path.realpath(__file__))
        with open('{}/../../sample_data/transactionDetail.json'.format(dir_path), encoding='utf_8_sig') as data_file:
            self._transaction_detail = json.loads(data_file.read())

    def test_from_raw(self):
        jq_transaction = JoinQuantTransaction(self._transaction_detail['data']['transaction'][0])
        transaction = jq_transaction.normalize()
        self.assertEqual(transaction.completed_at, datetime.strptime('2016-11-22 09:30', '%Y-%m-%d %H:%M'))
        self.assertEqual(transaction.action, 'BUY')
        self.assertEqual(transaction.symbol, '000001')
        self.assertEqual(transaction.price, 9.25)
        self.assertEqual(transaction.amount, 100)



# -*- coding: utf-8 -*-

import re
from datetime import datetime

from shipane_sdk.transaction import Transaction


class JoinQuantTransaction(object):
    def __init__(self, json):
        self.__dict__.update(json)

    def normalize(self):
        transaction = Transaction()
        transaction.completed_at = datetime.strptime('{} {}'.format(self.date, self.time), '%Y-%m-%d %H:%M')
        transaction.action = 'BUY' if self.transaction == u'买' else 'SELL'
        transaction.symbol = re.search(".*\\((\\d+)\\..*\\)", self.stock).group(1)
        transaction.price = self.price
        transaction.amount = int(re.search(u".*>[-]*(\\d+).*", self.amount, re.UNICODE).group(1))
        return transaction



import shipane_sdk


# 注意：需将回测调成分钟级别
# 注意：SDK内部取今日时间来获取新股数据
# 注意：用于回测没有意义，需挂到“我的交易”

def initialize(context):
    # 每天的开市后10分钟进行新股申购，参数设置见：https://www.joinquant.com/api#定时运行
    run_daily(purchase_new_stocks, '9:40')


def process_initialize(context):
    # 创建 JoinQuantExecutor 对象
    # 可选参数包括：host, port, key, client, timeout 等
    # 请将下面的 IP 替换为实际 IP
    g.__executor = shipane_sdk.JoinQuantExecutor(
        host='xxx.xxx.xxx.xxx',
        port=8888,
        key='',
        client=''
    )


def purchase_new_stocks(context):
    if context.run_params.type == 'sim_trade':
        g.__executor.purchase_new_stocks()
    else:
        log.info('回测中不进行新股申购')



# -*- coding: utf-8 -*-

import lxml.html.soupparser
import pandas as pd
import requests


class StockUtils(object):
    @staticmethod
    def new_stocks():
        url = 'http://vip.stock.finance.sina.com.cn/corp/view/vRPD_NewStockIssue.php?page=1&cngem=0&orderBy=NetDate&orderType=desc'
        request = requests.get(url)
        doc = lxml.html.soupparser.fromstring(request.content, features='html.parser')
        table = doc.cssselect('table#NewStockTable')[0]
        table.remove(table.cssselect('thead')[0])
        table_html = lxml.html.etree.tostring(table).decode('utf-8')
        df = pd.read_html(table_html, skiprows=[0, 1])[0]
        df = df.select(lambda x: x in [0, 1, 2, 3, 7], axis=1)
        df.columns = ['code', 'xcode', 'name', 'ipo_date', 'price']
        df['code'] = df['code'].map(lambda x: str(x).zfill(6))
        df['xcode'] = df['xcode'].map(lambda x: str(x).zfill(6))
        return df



# -*- coding: utf-8 -*-

from rqopen_client import RQOpenClient

from shipane_sdk.base_quant_client import BaseQuantClient
from shipane_sdk.ricequant.transaction import RiceQuantTransaction


class RiceQuantClient(BaseQuantClient):
    def __init__(self, **kwargs):
        super(RiceQuantClient, self).__init__('RiceQuant')

        self._rq_client = RQOpenClient(kwargs.get('username', None), kwargs.get('password', None),
                                       timeout=kwargs.pop('timeout', (5.0, 10.0)))
        self._run_id = kwargs.get('run_id', None)

    def login(self):
        self._rq_client.login(timeout=self._timeout)
        super(RiceQuantClient, self).login()

    def query(self):
        response = self._rq_client.get_day_trades(self._run_id, timeout=self._timeout)
        raw_transactions = response['resp']['trades']
        transactions = []
        for raw_transaction in raw_transactions:
            transaction = RiceQuantTransaction(raw_transaction).normalize()
            transactions.append(transaction)

        return transactions



# -*- coding: utf-8 -*-

import logging

import shipane_sdk

logging.basicConfig(level=logging.DEBUG)

client = shipane_sdk.Client(host='localhost', port=8888, key='')
account_info = client.get_account('title:monijiaoyi')
print(account_info)



# -*- coding: utf-8 -*-
import codecs
import os
import unittest

import six
from six.moves import configparser

if six.PY2:
    ConfigParser = configparser.RawConfigParser
else:
    ConfigParser = configparser.ConfigParser

from shipane_sdk.ricequant.client import RiceQuantClient


class RiceQuantClientTest(unittest.TestCase):
    def setUp(self):
        config = ConfigParser()
        dir_path = os.path.dirname(os.path.realpath(__file__))
        config.readfp(codecs.open('{}/../../config/config.ini'.format(dir_path), encoding="utf_8_sig"))
        self._rq_client = RiceQuantClient(**dict(config.items('RiceQuant')))

    def test_query(self):
        self._rq_client.login()
        transactions = self._rq_client.query()
        self.assertTrue(isinstance(transactions, list))



# -*- coding: utf-8 -*-

from apscheduler.triggers.cron import CronTrigger


class APCronParser(object):
    @classmethod
    def parse(cls, expression):
        parts = list(reversed(expression.split()))
        for i in range(len(parts)):
            if parts[i] == '?':
                parts[i] = None

        return CronTrigger(parts[0], parts[1], parts[2], parts[3], parts[4], parts[5], parts[6], parts[7])



# -*- coding: utf-8 -*-

from datetime import date, timedelta, datetime


class BaseQuantClient(object):
    def __init__(self, name):
        self._name = name
        self._last_login_time = datetime.now() - timedelta(1)

    @property
    def name(self):
        return self._name

    def login(self):
        self._last_login_time = datetime.now()

    def is_login(self):
        return self._last_login_time >= datetime.combine(date.today(), datetime.min.time())

    def query(self):
        return []



# -*- coding: utf-8 -*-

import datetime

import shipane_sdk


class RiceQuantExecutor(object):
    def __init__(self, **kwargs):
        try:
            self._logger = logger
        except NameError:
            import logging
            self._logger = logging.getLogger()
        self._client = shipane_sdk.Client(self._logger, **kwargs)
        self._client_param = kwargs.get('client')
        self._order_id_map = dict()
        self._expire_before = datetime.datetime.combine(datetime.date.today(), datetime.time.min)

    @property
    def client(self):
        return self._client

    def purchase_new_stocks(self):
        self.client.purchase_new_stocks(self._client_param)

    def execute(self, order_id):
        if order_id is None:
            self._logger.info('[实盘易] 委托编号为空，忽略下单请求')
            return

        try:
            order = get_order(order_id)
            self._logger.info("[实盘易] 跟单：" + str(order))

            if order is None:
                self._logger.info('[实盘易] 委托为空，忽略下单请求')
                return
            if self.__is_expired(order):
                self._logger.info('[实盘易] 委托已过期，忽略下单请求')
                return

            price_type = 0 if order.type.name == 'LIMIT' else 4
            actual_order = self._client.execute(self._client_param,
                                                action=order.side.name,
                                                symbol=order.order_book_id,
                                                type=order.type.name,
                                                priceType=price_type,
                                                price=order.price,
                                                amount=order.quantity)
            self._order_id_map[order_id] = actual_order['id']
            return actual_order
        except Exception as e:
            self._logger.error("[实盘易] 下单异常：" + str(e))

    def cancel(self, order_id):
        if order_id is None:
            self._logger.info('[实盘易] 委托为空，忽略撤单请求')
            return

        try:
            if order_id in self._order_id_map:
                self._client.cancel(self._client_param, self._order_id_map[order_id])
            else:
                self._logger.warning('[实盘易] 未找到对应的委托编号')
        except Exception as e:
            self._logger.error("[实盘易] 撤单异常：" + str(e))

    def __is_expired(self, order):
        return order.datetime < self._expire_before



# -*- coding: utf-8 -*-

from datetime import datetime

from shipane_sdk.transaction import Transaction


class UqerTransaction(object):
    def __init__(self, json):
        self.__dict__.update(json)

    def normalize(self):
        transaction = Transaction()
        transaction.completed_at = datetime.fromtimestamp(self.place_time / 1000.0)
        transaction.action = self.side
        transaction.symbol = self.ticker
        transaction.price = self.execution_avg_price
        transaction.amount = abs(self.amount)
        return transaction



# -*- coding: utf-8 -*-

from shipane_sdk.jobs.basic_job import BasicJob


class NewStockPurchaseJob(BasicJob):
    def __init__(self, client, client_aliases=None, name=None, **kwargs):
        super(NewStockPurchaseJob, self).__init__(name, kwargs.get('schedule', None), kwargs.get('enabled', False))

        self._client = client
        self._client_aliases = client_aliases

    def __call__(self):
        for client_alias in self._client_aliases:
            client = self._client_aliases[client_alias]
            self._client.purchase_new_stocks(client)



# -*- coding: utf-8 -*-

import copy
import datetime
import re

import lxml.html
import pandas as pd
import requests
import six
from lxml import etree
from pandas.compat import StringIO
from requests import Request
from six.moves.urllib.parse import urlencode


class Client(object):
    KEY_REGEX = r'key=([^&]*)'

    def __init__(self, logger=None, **kwargs):
        if logger is not None:
            self._logger = logger
        else:
            import logging
            self._logger = logging.getLogger(__name__)
        self._host = kwargs.pop('host', 'localhost')
        self._port = kwargs.pop('port', 8888)
        self._key = kwargs.pop('key', '')
        self._timeout = kwargs.pop('timeout', (5.0, 10.0))

    @property
    def host(self):
        return self._host

    @host.setter
    def host(self, value):
        self._host = value

    @property
    def port(self):
        return self._port

    @port.setter
    def port(self, value):
        self._port = value

    @property
    def key(self):
        return self._key

    @key.setter
    def key(self, value):
        self._key = value

    @property
    def timeout(self):
        return self._timeout

    @timeout.setter
    def timeout(self, value):
        self._timeout = value

    def get_statuses(self, timeout=None):
        request = Request('GET', self.__create_url(None, 'statuses'))
        self.__send_request(request, timeout)

    def get_account(self, client=None, timeout=None):
        request = Request('GET', self.__create_url(client, 'accounts'))
        response = self.__send_request(request, timeout)
        return response.json()

    def get_positions(self, client=None, timeout=None):
        request = Request('GET', self.__create_url(client, 'positions'))
        response = self.__send_request(request, timeout)
        json = response.json()
        sub_accounts = pd.DataFrame(json['subAccounts']).T
        positions = pd.DataFrame(json['dataTable']['rows'], columns=json['dataTable']['columns'])
        return {'sub_accounts': sub_accounts, 'positions': positions}

    def get_orders(self, client=None, status="", timeout=None):
        request = Request('GET', self.__create_url(client, 'orders', status=status))
        response = self.__send_request(request, timeout)
        json = response.json()
        df = pd.DataFrame(json['dataTable']['rows'], columns=json['dataTable']['columns'])
        return df

    def buy(self, client=None, timeout=None, **kwargs):
        kwargs['action'] = 'BUY'
        return self.__execute(client, timeout, **kwargs)

    def sell(self, client=None, timeout=None, **kwargs):
        kwargs['action'] = 'SELL'
        return self.__execute(client, timeout, **kwargs)

    def execute(self, client=None, timeout=None, **kwargs):
        return self.__execute(client, timeout, **kwargs)

    def cancel(self, client=None, order_id=None, timeout=None):
        request = Request('DELETE', self.__create_order_url(client, order_id))
        self.__send_request(request, timeout)

    def cancel_all(self, client=None, timeout=None):
        request = Request('DELETE', self.__create_order_url(client))
        self.__send_request(request, timeout)

    def query(self, client=None, navigation=None, timeout=None):
        request = Request('GET', self.__create_url(client, '', navigation=navigation))
        response = self.__send_request(request, timeout)
        json = response.json()
        df = pd.DataFrame(json['dataTable']['rows'], columns=json['dataTable']['columns'])
        return df

    def query_new_stocks(self):
        return self.__query_new_stocks()

    def purchase_new_stocks(self, client=None, timeout=None):
        today = datetime.datetime.strftime(datetime.datetime.today(), '%Y-%m-%d')
        df = self.query_new_stocks()
        df = df[(df.ipo_date == today)]
        self._logger.info('今日可申购新股有[%d]只', len(df))
        for index, row in df.iterrows():
            try:
                order = {
                    'symbol': row['xcode'], 'type': 'LIMIT', 'price': row['price'], 'amountProportion': 'ALL'
                }
                self._logger.info('申购新股：%s', str(order))
                self.buy(client, timeout, **order)
            except Exception as e:
                self._logger.error('客户端[%s]申购新股[%s(%s)]失败\n%s', client, row['name'], row['code'], e)

    def start_clients(self, timeout=None):
        request = Request('PUT', self.__create_url(None, 'clients'))
        self.__send_request(request, timeout)

    def shutdown_clients(self, timeout=None):
        request = Request('DELETE', self.__create_url(None, 'clients'))
        self.__send_request(request, timeout)

    def __execute(self, client=None, timeout=None, **kwargs):
        if not kwargs.get('type'):
            kwargs['type'] = 'LIMIT'
        request = Request('POST', self.__create_order_url(client), json=kwargs)
        response = self.__send_request(request)
        return response.json()

    def __query_new_stocks(self):
        DATA_URL = 'http://vip.stock.finance.sina.com.cn/corp/view/vRPD_NewStockIssue.php?page=1&cngem=0&orderBy=NetDate&orderType=desc'
        html = lxml.html.parse(DATA_URL)
        res = html.xpath('//table[@id=\"NewStockTable\"]/tr')
        if six.PY2:
            sarr = [etree.tostring(node) for node in res]
        else:
            sarr = [etree.tostring(node).decode('utf-8') for node in res]
        sarr = ''.join(sarr)
        sarr = sarr.replace('<font color="red">*</font>', '')
        sarr = '<table>%s</table>' % sarr
        df = pd.read_html(StringIO(sarr), skiprows=[0, 1])[0]
        df = df.select(lambda x: x in [0, 1, 2, 3, 7], axis=1)
        df.columns = ['code', 'xcode', 'name', 'ipo_date', 'price']
        df['code'] = df['code'].map(lambda x: str(x).zfill(6))
        df['xcode'] = df['xcode'].map(lambda x: str(x).zfill(6))
        return df

    def __create_order_url(self, client=None, order_id=None, **params):
        return self.__create_url(client, 'orders', order_id, **params)

    def __create_url(self, client, resource, resource_id=None, **params):
        all_params = copy.deepcopy(params)
        if client is not None:
            all_params.update(client=client)
        all_params.update(key=self._key)
        if resource_id is None:
            path = '/{}'.format(resource)
        else:
            path = '/{}/{}'.format(resource, resource_id)

        return '{}{}?{}'.format(self.__create_base_url(), path, urlencode(all_params))

    def __create_base_url(self):
        return 'http://' + self._host + ':' + str(self._port)

    def __send_request(self, request, timeout=None):
        prepared_request = request.prepare()
        timeout = timeout if timeout is not None else self._timeout
        self.__log_request(prepared_request)
        with requests.sessions.Session() as session:
            response = session.send(prepared_request, timeout=timeout)
        self.__log_response(response)
        response.raise_for_status()
        return response

    def __log_request(self, prepared_request):
        url = self.__eliminate_privacy(prepared_request.path_url)
        if prepared_request.body is None:
            self._logger.info('Request:\n%s %s', prepared_request.method, url)
        else:
            self._logger.info('Request:\n%s %s\n%s', prepared_request.method, url, prepared_request.body)

    def __log_response(self, response):
        message = 'Response:\n{} {}\n{}'.format(response.status_code, response.reason, response.text)
        if response.status_code == 200:
            self._logger.info(message)
        else:
            self._logger.error(message)

    @classmethod
    def __eliminate_privacy(cls, url):
        match = re.search(cls.KEY_REGEX, url)
        key = match.group(1)
        masked_key = '*' * len(key)
        url = re.sub(cls.KEY_REGEX, "key={}".format(masked_key), url)
        return url



# -*- coding: utf-8 -*-
import json
import os
import unittest
from datetime import datetime

from shipane_sdk.uqer.transaction import UqerTransaction


class TransactionTest(unittest.TestCase):
    def setUp(self):
        dir_path = os.path.dirname(os.path.realpath(__file__))
        with open('{}/../../sample_data/uqer-order-response.json'.format(dir_path), 'r',
                  encoding='utf_8_sig') as data_file:
            self._transaction_detail = json.loads(data_file.read())

    def test_from_raw(self):
        uq_transaction = UqerTransaction(self._transaction_detail[0])
        transaction = uq_transaction.normalize()
        self.assertEqual(transaction.completed_at,
                         datetime.strptime('2017-02-14 09:30:06.610000', '%Y-%m-%d %H:%M:%S.%f'))
        self.assertEqual(transaction.action, 'BUY')
        self.assertEqual(transaction.symbol, '000001')
        self.assertEqual(transaction.price, 9.42)
        self.assertEqual(transaction.amount, 100)



# -*- coding: utf-8 -*-
import json
import os
import unittest
from datetime import datetime

from shipane_sdk.ricequant.transaction import RiceQuantTransaction


class TransactionTest(unittest.TestCase):
    def setUp(self):
        dir_path = os.path.dirname(os.path.realpath(__file__))
        with open('{}/../../sample_data/rq_client-response.json'.format(dir_path), encoding='utf_8_sig') as data_file:
            self._transaction_detail = json.loads(data_file.read())

    def test_from_raw(self):
        rq_transaction = RiceQuantTransaction(self._transaction_detail['resp']['trades'][0])
        transaction = rq_transaction.normalize()
        self.assertEqual(transaction.completed_at, datetime.strptime('2016-12-23 09:32:00', '%Y-%m-%d %H:%M:%S'))
        self.assertEqual(transaction.action, 'SELL')
        self.assertEqual(transaction.symbol, '600216.XSHG')
        self.assertEqual(transaction.price, 12.77)
        self.assertEqual(transaction.amount, 100)



# -*- coding: utf-8 -*-

import logging
import os
import unittest

import six
from requests import HTTPError
from six.moves import configparser

from shipane_sdk import Client

if six.PY2:
    ConfigParser = configparser.RawConfigParser
else:
    ConfigParser = configparser.ConfigParser


class ClientTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        logging.basicConfig(level=logging.DEBUG)
        config = ConfigParser()
        dir_path = os.path.dirname(os.path.realpath(__file__))
        config.read('{}/../config/config.ini'.format(dir_path))
        cls.client = Client(logging.getLogger(), **dict(config.items('ShiPanE')))
        cls.client_param = config.get('ShiPanE', 'client')
        cls.client.start_clients()

    def test_get_account(self):
        try:
            self.client.get_account(self.client_param)
        except HTTPError as e:
            self.fail()

    def test_get_positions(self):
        try:
            data = self.client.get_positions(self.client_param)
            sub_accounts = data['sub_accounts']
            self.assertGreaterEqual(sub_accounts['余额']['人民币'], 0)
            positions = data['positions']
            self.assertIsNotNone(positions['证券代码'][0])
        except HTTPError as e:
            self.fail()

    def test_get_orders(self):
        try:
            df = self.client.get_orders(self.client_param)
            self.assertIsNotNone(df['委托编号'][0])
        except HTTPError as e:
            self.fail()

    def test_get_open_orders(self):
        try:
            df = self.client.get_orders(self.client_param, 'open')
            self.assertIsNotNone(df['委托编号'][0])
        except HTTPError as e:
            self.fail()

    def test_get_filled_orders(self):
        try:
            df = self.client.get_orders(self.client_param, 'filled')
            self.assertIsNotNone(df['委托编号'][0])
        except HTTPError as e:
            self.fail()

    def test_buy_stock(self):
        try:
            order = self.client.buy(self.client_param, symbol='000001', price=8.11, amount=100)
            self.assertIsNotNone(order['id'])
        except HTTPError as e:
            result = e.response.json()
            self.assertIsNotNone(result['message'])

    def test_sell_stock(self):
        try:
            order = self.client.sell(self.client_param, symbol='000001', price=9.51, amount=100)
            self.assertIsNotNone(order['id'])
        except HTTPError as e:
            result = e.response.json()
            self.assertIsNotNone(result['message'])

    def test_cancel_all(self):
        try:
            self.client.cancel_all(self.client_param)
        except HTTPError as e:
            self.fail()

    def test_query(self):
        try:
            df = self.client.query(self.client_param, '查询>资金股份')
            self.assertIsNotNone(df['证券代码'][0])
        except HTTPError as e:
            self.fail()

    def test_query_new_stocks(self):
        df = self.client.query_new_stocks()
        self.assertTrue((df.columns == ['code', 'xcode', 'name', 'ipo_date', 'price']).all())
